"""This module is for the translation mapping data of the
following document:

Document file: text_align.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# text_align property": "# text_align 属性",
    ##################################################
    "This page explains the text-related `text_align` property.": "このページではテキスト関係の`text_align`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `text_align` property updates or gets the instance's text alignment setting.": "`text_align`属性ではインスタンスのテキストの行揃えの設定値の更新もしくは取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter and setter interfaces' type becomes a `CssTextAlign` enum value.": "getterとsetterの各インターフェイスの値は`CssTextAlign`の型のenumの値となります。",  # noqa
    ##################################################
    "The acceptable enum values are as follows:": "設定できるenumの値は以下の通りです:",
    ##################################################
    "- `CssTextAlign.LEFT` (default)": "- `CssTextAlign.LEFT`（デフォルト）",
    ##################################################
    "- `CssTextAlign.CENTER`": "- `CssTextAlign.CENTER`",
    ##################################################
    "- `CssTextAlign.RIGHT`": "- `CssTextAlign.RIGHT`",
    ##################################################
    "- `CssTextAlign.JUSTIFY`": "- `CssTextAlign.JUSTIFY",
    ##################################################
    "## Example of CssTextAlign.LEFT": "## CssTextAlign.LEFTの例",
    ##################################################
    "Notes: This setting is a default setting.": "特記事項 : この設定はデフォルトの設定です。",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.LEFT\n\nap.save_overall_html(dest_dir_path="./css_text_align_left/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.LEFT\n\nap.save_overall_html(dest_dir_path="./css_text_align_left/")\n```',  # noqa
    ##################################################
    "## Example of CssTextAlign.CENTER": "## CssTextAlign.CENTERの例",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.CENTER\n\nap.save_overall_html(dest_dir_path="./css_text_align_center/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.CENTER\n\nap.save_overall_html(dest_dir_path="./css_text_align_center/")\n```',  # noqa
    ##################################################
    "## Example of CssTextAlign.RIGHT": "## CssTextAlign.RIGHTの例",
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.RIGHT\n\nap.save_overall_html(dest_dir_path="./css_text_align_right/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.RIGHT\n\nap.save_overall_html(dest_dir_path="./css_text_align_right/")\n```',  # noqa
    ##################################################
    "## Example of CSSTextAlign.JUSTIFY": "## CssTextAlign.JUSTIFYの例",
    ##################################################
    "Notes: This enum setting justifies a text evenly.": "特記じこを : このenumの設定はテキストを均等配置します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.JUSTIFY\n\nap.save_overall_html(dest_dir_path="./css_text_align_justify/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.text_align = ap.CssTextAlign.JUSTIFY\n\nap.save_overall_html(dest_dir_path="./css_text_align_justify/")\n```',  # noqa
    ##################################################
    "## text_align property API": "## text_align 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a text-align value.<hr>": "テキストの行揃え設定の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `text_align`: CssTextAlign": "- `text_align`: CssTextAlign",
    ##################################################
    "  - A text-align value.": "  - 行揃え設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=170,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.text_align = ap.CssTextAlign.RIGHT\n>>> assert multi_line_text.text_align == ap.CssTextAlign.RIGHT\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=170,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.text_align = ap.CssTextAlign.RIGHT\n>>> assert multi_line_text.text_align == ap.CssTextAlign.RIGHT\n```',  # noqa
}



"""Mix-in class implementation for the `_set_initial_y` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number
from apysc._validation import arg_validation_decos


class SetInitialYMixIn:
    _y: Number

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_y(
        self,
        *,
        y: Union[float, Number],
        variable_name_suffix: str = "",
    ) -> None:
        """
        Set a chart's initial y value.

        Parameters
        ----------
        y : Union[float, Number]
            A chart's initial y-coordinate value.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        if not isinstance(y, Number):
            y_: Number = Number(y, variable_name_suffix=variable_name_suffix)
        else:
            y_ = y
        self._y = y_



"""Class implementation for the timer.
"""

from typing import TYPE_CHECKING
from typing import Any
from typing import Callable
from typing import Optional
from typing import TypeVar
from typing import Union

from typing_extensions import final

from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._event.handler import HandlerData
from apysc._html.debug_mode import add_debug_info_setting
from apysc._time.fps import FPS
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.number_value_mixin import NumberValueMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._event.timer_event import TimerEvent

_ConstructorOptions = TypeVar("_ConstructorOptions")
_TimerCompleteOptions = TypeVar("_TimerCompleteOptions")
_Handler = Callable[["TimerEvent", _ConstructorOptions], None]


class Timer(VariableNameMixIn, CustomEventMixIn["TimerEvent"]):
    """
    Timer class to handle function calling at regular intervals.

    References
    ----------
    - Timer
        - https://simon-ritchie.github.io/apysc/en/timer.html
    - TimerEvent class
        - https://simon-ritchie.github.io/apysc/en/timer_event.html
    - Timer class delay setting
        - https://simon-ritchie.github.io/apysc/en/timer_delay.html
    - FPS enum
        - https://simon-ritchie.github.io/apysc/en/fps.html
    - Timer class repeat_count setting
        - https://simon-ritchie.github.io/apysc/en/timer_repeat_count.html

    Examples
    --------
    >>> from typing_extensions import TypedDict
    >>> import apysc as ap
    >>> _ = ap.Stage()
    >>> class RectOptions(TypedDict):
    ...     rectangle: ap.Rectangle
    ...
    >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
    ...     rectangle: ap.Rectangle = options["rectangle"]
    ...     rectangle.x += 1
    >>> def on_timer_complete(e: ap.TimerEvent, options: dict) -> None:
    ...     ap.trace("Timer completed!")
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50
    ... )
    >>> options: RectOptions = {"rectangle": rectangle}
    >>> timer: ap.Timer = ap.Timer(
    ...     on_timer, delay=ap.FPS.FPS_60, repeat_count=50, options=options
    ... )
    >>> _ = timer.timer_complete(on_timer_complete)
    >>> timer.start()
    """

    _delay: Number
    _repeat_count: Int
    _current_count: Int
    _handler_data: HandlerData["TimerEvent"]
    _handler_name: str
    _running: Boolean

    @final
    @arg_validation_decos.handler_args_num(arg_position_index=1)
    @arg_validation_decos.is_integer(arg_position_index=3, optional=False)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=3, optional=False)
    @arg_validation_decos.handler_options_type(arg_position_index=4)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        handler: _Handler[_ConstructorOptions],
        *,
        delay: Union[int, float, NumberValueMixIn, FPS],
        repeat_count: Union[int, Int] = 0,
        options: Optional[_ConstructorOptions] = None,
    ) -> None:
        """
        Timer class to handle function calling at regular intervals.

        Parameters
        ----------
        handler : _Handler
            A handler would be called at regular intervals.
        delay : Int or int or Number or float or FPS
            A delay between each `Handler` calling in a millisecond
            or FPS value. If an `FPS` value is specified, this
            value becomes a millisecond calculated with that
            FPS value (e.g., if the `FPS_60` value is specified,
            then `delay` becomes 16.6666667).
        repeat_count : Int or int
            Max count of a `Handler`'s calling. A timer stops
            if the `Handler`'s calling count has reached this value.
            If 0 is specified, then a timer loops forever.
        options : dict or None, default None
            Optional arguments dictionary to pass a `Handler` callable.

        References
        ----------
        - Timer
            - https://simon-ritchie.github.io/apysc/en/timer.html
        - TimerEvent class
            - https://simon-ritchie.github.io/apysc/en/timer_event.html
        - Timer class delay setting
            - https://simon-ritchie.github.io/apysc/en/timer_delay.html
        - FPS enum
            - https://simon-ritchie.github.io/apysc/en/fps.html
        - Timer class repeat_count setting
            - https://simon-ritchie.github.io/apysc/en/timer_repeat_count.html
        - About the handler options' type
            - https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html  # noqa

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        ...
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options["rectangle"]
        ...     rectangle.x += 1
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> options: RectOptions = {"rectangle": rectangle}
        >>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()
        """
        from apysc._event.handler import get_handler_name
        from apysc._expression import expression_data_util
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope
        from apysc._validation import number_validation

        with TemporaryNotHandlerScope():
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=var_names.TIMER
            )
            self._handler_name = get_handler_name(handler=handler, instance=self)
            handler = self._wrap_handler(handler=handler)
            self._handler: _Handler[_ConstructorOptions] = handler  # type: ignore
            delay_: Number = self._convert_delay_to_number(delay=delay)
            number_validation.validate_num_is_gte_zero(num=delay_)
            self._delay = delay_
            if isinstance(repeat_count, Int):
                repeat_count_: Int = repeat_count
            else:
                repeat_count_ = Int(repeat_count)
            self._repeat_count = repeat_count_
            self._running = Boolean(False)
            self._current_count = Int(0)
            if options is None:
                options = {}  # type: ignore
            self._handler_data = HandlerData(
                handler=self._handler,
                options=options,
            )
            expression_data_util.append_js_expression(
                expression=f"var {self.variable_name};"
            )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _convert_delay_to_number(
        self, *, delay: Union[int, float, NumberValueMixIn, FPS]
    ) -> Number:
        """
        Convert each type of delay value to a `Number` value.

        Parameters
        ----------
        delay : int or float or Int or Number or FPS
            A delay between each handler's calling in milliseconds
            or FPS value.

        Returns
        -------
        delay_ : Number
            Converted delay value.
        """
        from apysc._time.fps import FPSDefinition

        if isinstance(delay, FPS):
            fps_definition: FPSDefinition = delay.value
            delay = fps_definition.millisecond_interval
        if not isinstance(delay, Number):
            delay_: Number = Number(delay)
        else:
            delay_ = delay
        return delay_

    @property
    @add_debug_info_setting(module_name=__name__)
    def delay(self) -> Number:
        """
        Get a delay value.

        Returns
        -------
        delay : Number
            A delay value of each `Handler` calling in milliseconds.

        References
        ----------
        - Timer class delay setting
            - https://simon-ritchie.github.io/apysc/en/timer_delay.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
        ...     pass
        >>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)
        >>> timer.delay
        Number(33.3)
        """
        from apysc._type import value_util

        return value_util.get_copy(value=self._delay)

    @property
    @add_debug_info_setting(module_name=__name__)
    def repeat_count(self) -> Int:
        """
        Get a max count value of a handler's calling.

        Returns
        -------
        repeat_count : Int
            Max count of a handler's calling.
            If this value is 0, then a timer loop forever.

        References
        ----------
        - Timer class repeat_count setting
            - https://simon-ritchie.github.io/apysc/en/timer_repeat_count.html

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
        ...     pass
        >>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)
        >>> timer.repeat_count
        Int(50)
        """
        from apysc._type import value_util

        return value_util.get_copy(value=self._repeat_count)

    @property
    @add_debug_info_setting(module_name=__name__)
    def running(self) -> Boolean:
        """
        Get a boolean value whether this timer is running or not.

        Returns
        -------
        running : Boolean.
            A boolean value whether this timer is running or not.

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
        ...     pass
        >>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, repeat_count=50)
        >>> timer.running
        Boolean(False)

        >>> timer.start()
        >>> timer.running
        Boolean(True)
        """
        from apysc._type import value_util

        return value_util.get_copy(value=self._running)

    @property
    @add_debug_info_setting(module_name=__name__)
    def current_count(self) -> Int:
        """
        Get a current timer count.

        Returns
        -------
        current_count : Int
            A current timer count.

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
        ...     timer: ap.Timer = e.this
        ...     ap.trace(timer.current_count)
        >>> _ = ap.Timer(on_timer, delay=33.3, repeat_count=50).start()
        """
        from apysc._type import value_util

        return value_util.get_copy(value=self._current_count)

    @final
    @add_debug_info_setting(module_name=__name__)
    def start(self) -> None:
        """
        Start this timer.

        References
        ----------
        - Timer class start and stop interfaces
            - https://simon-ritchie.github.io/apysc/en/timer_start_and_stop.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
        ...     pass
        >>> _ = ap.Timer(on_timer, delay=33.3, repeat_count=50).start()
        """
        from apysc._event import handler_circular_calling_util
        from apysc._event.handler import append_handler_expression
        from apysc._event.timer_event import TimerEvent
        from apysc._expression import expression_data_util
        from apysc._type import value_util

        delay_val_str: str = value_util.get_value_str_for_expression(value=self._delay)
        is_handler_circular_calling: bool = (
            handler_circular_calling_util.is_handler_circular_calling(
                handler_name=self._handler_name
            )
        )
        if is_handler_circular_calling:
            handler_name: str = handler_circular_calling_util.get_prev_handler_name(
                handler_name=self._handler_name
            )
            variable_name: str = handler_circular_calling_util.get_prev_variable_name(
                handler_name=self._handler_name
            )
        else:
            handler_name = self._handler_name
            variable_name = self.variable_name
        expression: str = (
            f"\nif (_.isUndefined({variable_name})) {{"
            f"\n  {variable_name} = setInterval("
            f"{handler_name}, {delay_val_str});"
            "\n}"
        )
        expression_data_util.append_js_expression(expression=expression)

        e: TimerEvent = TimerEvent(this=self)
        append_handler_expression(
            handler_data=self._handler_data, handler_name=handler_name, e=e
        )
        self._running.value = True

    @final
    def _wrap_handler(
        self,
        *,
        handler: _Handler[_ConstructorOptions],
    ) -> _Handler[_ConstructorOptions]:
        """
        Wrap a handler to update a current count value when
        it is called.

        Parameters
        ----------
        handler : _Handler
            Target handler.

        Returns
        -------
        wrapped : _Handler
            A wrapped handler.
        """
        from apysc._event.timer_event import TimerEvent

        def wrapped(e: TimerEvent, options: Any) -> None:
            """
            Wrapped handler.

            Parameters
            ----------
            e : TimerEvent
                Event instance.
            options : dict
                Optional arguments dictionary.
            """
            e.this._current_count += 1
            e.this._current_count._value = 0
            handler(e, options)
            self._append_count_branch_expression()

        return wrapped

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_count_branch_expression(self) -> None:
        """
        Append the timer stopping expression by the counting.
        """
        from apysc._event.custom_event_type import CustomEventType
        from apysc._expression import expression_data_util
        from apysc._expression.indent_num import Indent
        from apysc._type import value_util

        current_count_val_str: str = value_util.get_value_str_for_expression(
            value=self._current_count
        )
        repeat_count_val_str: str = value_util.get_value_str_for_expression(
            value=self._repeat_count
        )
        expression: str = (
            f"if ({repeat_count_val_str} !== 0 "
            f"&& {current_count_val_str} === {repeat_count_val_str}) {{"
            f"\n{self._get_stop_expression(indent_num=1)}"
        )
        expression_data_util.append_js_expression(expression=expression)
        with Indent():
            self._running.value = False
            self.trigger_custom_event(custom_event_type=CustomEventType.TIMER_COMPLETE)
            self.unbind_custom_event_all(
                custom_event_type=CustomEventType.TIMER_COMPLETE
            )
            self._current_count.value = 0
        expression = "\n}"
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def stop(self) -> None:
        """
        Stop this timer.

        References
        ----------
        - Timer class start and stop interfaces
            - https://simon-ritchie.github.io/apysc/en/timer_start_and_stop.html  # noqa

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        ...
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options["rectangle"]
        ...     rectangle.x += 1
        ...     with ap.If(rectangle.x > 100):
        ...         timer: ap.Timer = e.this
        ...         timer.stop()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> options: RectOptions = {"rectangle": rectangle}
        >>> _ = ap.Timer(on_timer, delay=33.3, options=options).start()
        """
        from apysc._expression import expression_data_util

        self._running.value = False
        expression: str = self._get_stop_expression(indent_num=0)
        expression_data_util.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _get_stop_expression(self, *, indent_num: int) -> str:
        """
        Get a stop interface expression string.

        Parameters
        ----------
        indent_num : int
            Indentation number to append.

        Returns
        -------
        expression : str
            Stop interface expression string.
        """
        from apysc._string import indent_util

        expression: str = (
            f"if (!_.isUndefined({self.variable_name})) {{"
            f"\n  clearInterval({self.variable_name});"
            f"\n  {self.variable_name} = undefined;"
            "\n}"
        )
        expression = indent_util.append_spaces_to_expression(
            expression=expression, indent_num=indent_num
        )
        return expression

    @final
    @add_debug_info_setting(module_name=__name__)
    def reset(self) -> None:
        """
        Reset the timer count and stop this timer.

        References
        ----------
        - Timer class reset interface
            - https://simon-ritchie.github.io/apysc/en/timer_reset.html

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        ...
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options["rectangle"]
        ...     rectangle.x += 1
        ...     with ap.If(rectangle.x > 100):
        ...         timer: ap.Timer = e.this
        ...         timer.reset()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> options: RectOptions = {"rectangle": rectangle}
        >>> _ = ap.Timer(on_timer, delay=33.3, options=options).start()
        """
        self.stop()
        self._current_count.value = 0

    @final
    @arg_validation_decos.handler_args_num(arg_position_index=1)
    @arg_validation_decos.handler_options_type(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def timer_complete(
        self,
        handler: _Handler[_TimerCompleteOptions],
        *,
        options: Optional[_TimerCompleteOptions] = None,
    ) -> str:
        """
        Add a timer complete event listener setting.

        Parameters
        ----------
        handler : _Handler
            A callable that a timer calls when complete.
        options : dict or None, default None
            Optional arguments dictionary to be passed to a handler.

        Returns
        -------
        name : str
            Handler's name.

        References
        ----------
        - Timer class timer_complete interface
            - https://simon-ritchie.github.io/apysc/en/timer_complete.html
        - About the handler options' type
            - https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html  # noqa

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> _ = ap.Stage()
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        ...
        >>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options["rectangle"]
        ...     rectangle.x += 1
        >>> def on_timer_complete(e: ap.TimerEvent, options: RectOptions) -> None:
        ...     ap.trace("Timer completed!")
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> options: RectOptions = {"rectangle": rectangle}
        >>> timer: ap.Timer = ap.Timer(on_timer, delay=33.3, options=options)
        >>> _ = timer.timer_complete(on_timer_complete)
        >>> timer.start()
        """
        from apysc._event.custom_event_type import CustomEventType
        from apysc._event.timer_event import TimerEvent

        e: TimerEvent = TimerEvent(this=self)
        name: str = self.bind_custom_event(
            custom_event_type=CustomEventType.TIMER_COMPLETE,
            handler=handler,
            e=e,
            options=options,
        )
        return name



"""The class implementation for the `bug_report` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBugReportIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `bug_report` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M20 8h-2.81c-.45-.78-1.07-1.45-1.82-1.96L17 4.41 15.59 3l-2.17 2.17C12.96 5.06 12.49 5 12 5c-.49 0-.96.06-1.41.17L8.41 3 7 4.41l1.62 1.63C7.88 6.55 7.26 7.22 6.81 8H4v2h2.09c-.05.33-.09.66-.09 1v1H4v2h2v1c0 .34.04.67.09 1H4v2h2.81c1.04 1.79 2.97 3 5.19 3s4.15-1.21 5.19-3H20v-2h-2.09c.05-.33.09-.66.09-1v-1h2v-2h-2v-1c0-.34-.04-.67-.09-1H20V8zm-6 8h-4v-2h4v2zm0-4h-4v-2h4v2z"/></svg>'  # noqa



"""The mix-in class implementation for the `SvgText`'s `_set_delta_x` method.
"""

from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class SvgTextSetDeltaXMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_delta_x(self, *, delta_x: Union[float, Number]) -> None:
        """
        Set a coordinate delta-x value.

        Parameters
        ----------
        delta_x : Union[float, Number]
            A delta-x value.
        """
        from apysc._display.svg_text_delta_x_mixin import SvgTextDeltaXMixIn
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        if not isinstance(self, SvgTextDeltaXMixIn):
            raise TypeError(
                f"This method is only supported an {SvgTextDeltaXMixIn.__name__} "
                f"instance: {type(self).__name__}"
            )

        if not isinstance(delta_x, Number):
            suffix: str = get_attr_or_variable_name_suffix(
                instance=self, value_identifier="delta_x"
            )
            delta_x_: Number = Number(delta_x, variable_name_suffix=suffix)
        else:
            delta_x_ = delta_x
        self.delta_x = delta_x_



"""The mix-in class for the `_add_to_parent` method.
"""

from typing import Optional

from typing_extensions import final

from apysc._display.child_mixin import ChildMixIn
from apysc._display.display_object import DisplayObject
from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos


class AddToParentMixIn:
    @final
    @arg_validation_decos.is_display_object_container(
        arg_position_index=1, optional=True
    )
    @add_debug_info_setting(module_name=__name__)
    def _add_to_parent(self, *, parent: Optional[ChildMixIn]) -> None:
        """
        Add this instance to a specified parent instance.

        Parameters
        ----------
        parent : Optional[ChildMixIn]
            A parent instance. If the specified value is None,
            this interface uses a stage instance.
        """
        from apysc._display.stage import get_stage

        if not isinstance(self, DisplayObject):
            raise TypeError(
                "This method can only be called by a DisplayObject class instance."
            )

        if parent is None:
            parent = get_stage()
        parent.add_child(child=self)



"""The class implementation for the `forward_5` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialForward5Icon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `forward_5` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M18,13c0,3.31-2.69,6-6,6s-6-2.69-6-6s2.69-6,6-6v4l5-5l-5-5v4c-4.42,0-8,3.58-8,8c0,4.42,3.58,8,8,8c4.42,0,8-3.58,8-8 H18z"/><path d="M12.03,15.38c-0.44,0-0.58-0.31-0.6-0.56h-0.84c0.03,0.85,0.79,1.25,1.44,1.25c0.93,0,1.44-0.63,1.44-1.43 c0-1.33-0.97-1.44-1.3-1.44c-0.2,0-0.43,0.05-0.64,0.16l0.11-0.92h1.7v-0.71h-2.39l-0.25,2.17l0.67,0.17 c0.13-0.13,0.28-0.23,0.57-0.23c0.4,0,0.69,0.23,0.69,0.75C12.62,14.64,12.65,15.38,12.03,15.38z"/></g></g></svg>'  # noqa



"""The class implementation for the `unsubscribe` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialUnsubscribeIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `unsubscribe` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M18.5 13c-1.93 0-3.5 1.57-3.5 3.5s1.57 3.5 3.5 3.5 3.5-1.57 3.5-3.5-1.57-3.5-3.5-3.5zm2 4h-4v-1h4v1zm-6.95 0c-.02-.17-.05-.33-.05-.5 0-2.76 2.24-5 5-5 .92 0 1.76.26 2.5.69V5c0-1.1-.9-2-2-2H5c-1.1 0-2 .9-2 2v10c0 1.1.9 2 2 2h8.55zM12 10.5L5 7V5l7 3.5L19 5v2l-7 3.5z"/></svg>'  # noqa



"""The class implementation for the outlined `queue_music` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialQueueMusicOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `queue_music` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><rect fill="none" height="24" width="24"/><rect fill="none" height="24" width="24"/></g><g><g/><g><g transform="matrix(1 0 0 1 144 120)"><path d="M-122-114h-5v8.18c-0.31-0.11-0.65-0.18-1-0.18c-1.66,0-3,1.34-3,3s1.34,3,3,3s3-1.34,3-3v-9h3V-114 L-122-114z M-129-114h-12v2h12V-114L-129-114z M-129-110h-12v2h12V-110L-129-110z M-133-106h-8v2h8V-106L-133-106z M-129-103 c0-0.55,0.45-1,1-1c0.55,0,1,0.45,1,1s-0.45,1-1,1C-128.55-102-129-102.45-129-103z"/></g></g></g></svg>'  # noqa



"""The mix-in class implementation for the CSS text-align property.
"""

from apysc._display.css_text_align import CssTextAlign
from apysc._validation import arg_validation_decos


class TextAlignCssMixIn:
    _css_text_align: CssTextAlign = CssTextAlign.LEFT

    @property
    def text_align(self) -> CssTextAlign:
        """
        Get a text-align value.

        Returns
        -------
        text_align : CssTextAlign
            A text-align value.

        References
        ----------
        - text_align property
            - https://simon-ritchie.github.io/apysc/en/text_align.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=350,
        ...     stage_height=170,
        ...     stage_elem_id="stage",
        ... )
        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=300,
        ...     font_size=16,
        ...     fill_color=ap.Color("#00aaff"),
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.text_align = ap.CssTextAlign.RIGHT
        >>> assert multi_line_text.text_align == ap.CssTextAlign.RIGHT
        """
        return self._css_text_align

    @text_align.setter
    @arg_validation_decos.is_css_text_align(arg_position_index=1)
    def text_align(self, value: CssTextAlign) -> None:
        """
        Set a text-align value.

        Parameters
        ----------
        value : CssTextAlign
            A text-align value.

        References
        ----------
        - text_align property
            - https://simon-ritchie.github.io/apysc/en/text_align.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"),
        ...     stage_width=350,
        ...     stage_height=170,
        ...     stage_elem_id="stage",
        ... )
        >>> multi_line_text: ap.MultiLineText = ap.MultiLineText(
        ...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
        ...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        ...     "Ut enim ad minim veniam",
        ...     width=300,
        ...     font_size=16,
        ...     fill_color=ap.Color("#00aaff"),
        ...     x=25,
        ...     y=25,
        ... )
        >>> multi_line_text.text_align = ap.CssTextAlign.RIGHT
        >>> assert multi_line_text.text_align == ap.CssTextAlign.RIGHT
        """
        from apysc._display.css_interface import CssInterface
        from apysc._validation import display_validation

        self._css_text_align = value
        interface: CssInterface = display_validation.validate_css_interface(
            instance=self
        )
        interface.set_css(name="text-align", value=value.value)



"""The class implementation for the outlined `plagiarism` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPlagiarismOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `plagiarism` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M14,2H6C4.9,2,4,2.9,4,4v16c0,1.1,0.89,2,1.99,2H18c1.1,0,2-0.9,2-2V8L14,2z M18,20L6,20V4h7v5h5V20z"/><path d="M9.03,11.03c-1.37,1.37-1.37,3.58,0,4.95c1.12,1.12,2.8,1.31,4.13,0.59l1.88,1.88l1.41-1.41l-1.88-1.88 c0.71-1.33,0.53-3.01-0.59-4.13C12.61,9.66,10.39,9.66,9.03,11.03z M12.56,14.56c-0.59,0.59-1.54,0.59-2.12,0 c-0.59-0.59-0.59-1.54,0-2.12c0.59-0.59,1.54-0.59,2.12,0C13.15,13.03,13.15,13.97,12.56,14.56z"/></g></g></svg>'  # noqa



"""The mix-in class for the String's `zfill` method.
"""

from typing import TYPE_CHECKING
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._type.int import Int
    from apysc._type.string import String


class StringZfillMixIn:
    @final
    @add_debug_info_setting(module_name=__name__)
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    def zfill(self, *, width: Union[int, "Int"]) -> "String":
        """
        Return a copy of the string left filled with 0.

        Parameters
        ----------
        width : Union[int, "Int"]
            A width (length) of the string.

        Returns
        -------
        result : String
            A result string.

        References
        ----------
        - String class zfill method
            - https://simon-ritchie.github.io/apysc/en/string_zfill.html

        Examples
        --------
        >>> import apysc as ap
        >>> _: ap.Stage = ap.Stage()
        >>> string: ap.String = ap.String("1")
        >>> string = string.zfill(width=1)
        >>> string
        String("1")

        >>> string = string.zfill(width=3)
        >>> string
        String("001")

        >>> string = string.zfill(width=5)
        >>> string
        String("00001")
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_int_from_builtin_val,
        )
        from apysc._expression.expression_data_util import append_js_expression
        from apysc._type.int import Int
        from apysc._type.string import String
        from apysc._validation import string_validation

        self_string: String = string_validation.validate_apysc_string_type(
            string=self,
        )
        result: String = self_string.copy()
        width_: Int = get_copied_int_from_builtin_val(integer=width)
        result._value = result._value.zfill(width_._value)

        expression: str = (
            f"{result.variable_name} = {self_string.variable_name}"
            f'.padStart({width_.variable_name}, "0");'
        )
        append_js_expression(expression=expression)

        return result



"""The class implementation for the outlined `fiber_pin` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFiberPinOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `fiber_pin` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M11 15h1.5V9H11v6zm7.75-6v3.5L16.25 9H15v6h1.25v-3.5L18.8 15H20V9h-1.25zM7.5 9H4v6h1.5v-2h2c.85 0 1.5-.65 1.5-1.5v-1C9 9.65 8.35 9 7.5 9zm0 2.5h-2v-1h2v1z"/></svg>'  # noqa



"""Class implementation for the JavaScript expression string.
"""

from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._type.revert_mixin import RevertMixIn


class ExpressionString(RevertMixIn):
    """
    This class is for the JavaScript expression string.
    """

    _value: str

    @final
    def __init__(self, value: str) -> None:
        """
        The class for the JavaScript expression string.

        Parameters
        ----------
        value : str
            String to set.
        """
        self._value = value

    _value_snapshots: Optional[Dict[str, str]] = None

    @property
    def value(self) -> str:
        """
        Get a expression string.

        Returns
        -------
        value : str
            A expression string.
        """
        return self._value

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_value_snapshots", value=self._value, snapshot_name=snapshot_name
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._value = self._get_snapshot_val_if_exists(
            current_value=self._value,
            snapshot_dict=self._value_snapshots,
            snapshot_name=snapshot_name,
        )



"""This module is for the translation mapping data of the
following document:

Document file: for_dict_keys.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# ForDictKeys class": "# ForDictKeys クラス",
    ##################################################
    "This page explains the `ForDictKeys` class.": "このページでは`ForDictKeys`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page (the apysc uses this class for the same reason for each data type):": "事前に以下のページを読んでおくと役立つかもしれません（apyscライブラリではこのクラスを各データ型と同じように扱っています）。",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What class is this?": "## クラス概要",
    ##################################################
    "The `ForDictKeys` class is the for-loop class.": "`ForDictKeys`クラスはfor文でのループのためのクラスです。",  # noqa
    ##################################################
    "This interface returns `Dictionary`'s key in a loop.": "このインターフェイスはループ内で`Dictionary`のキーを返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "This class requires using the `with`-statement.": "このクラスは`with`ステートメントと共に使う必要があります。",  # noqa
    ##################################################
    "The `as`-keyword value becomes a `Dictionary`'s key.": "`as`キーワードの値は`Dictionary`のキーとなります。",  # noqa
    ##################################################
    "Also, this class requires the `dict_key_type` to specify a type of `Dictionary`'s key type.": "また、このクラスは`Dictionary`のキーの型を指定するための`dict_key_type`引数の指定が必要になります。",  # noqa
    ##################################################
    "This type only accepts an apysc type, such as the `String`, `Int`, `Number`, or `Boolean`.": "この型の指定は`String`や`Int`、`Number`、`Boolean`といったapyscの型のみ受け付けます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_: ap.Dictionary[ap.String, int] = ap.Dictionary(\n    {\n        ap.String("apple"): 120,\n        ap.String("orange"): 200,\n    }\n)\nkeys: ap.Array[ap.String] = ap.Array([])\nwith ap.ForDictKeys(dict_=dict_, dict_key_type=ap.String) as key:\n    keys.append(key)\nap.assert_arrays_equal(\n    keys,\n    ["apple", "orange"],\n)\n\nap.save_overall_html(dest_dir_path="for_dict_keys_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\n\ndict_: ap.Dictionary[ap.String, int] = ap.Dictionary(\n    {\n        ap.String("apple"): 120,\n        ap.String("orange"): 200,\n    }\n)\nkeys: ap.Array[ap.String] = ap.Array([])\nwith ap.ForDictKeys(dict_=dict_, dict_key_type=ap.String) as key:\n    keys.append(key)\nap.assert_arrays_equal(\n    keys,\n    ["apple", "orange"],\n)\n\nap.save_overall_html(dest_dir_path="for_dict_keys_basic_usage_1/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Each branch instruction class's scope variables reverting setting](branch_instruction_variables_reverting_setting.md)": "- [分岐条件の各クラスのスコープ内変数の復元設定](jp_branch_instruction_variables_reverting_setting.md)",  # noqa
    ##################################################
    "  - Notes: This class also has the same arguments and behaves in the same way.": "  - 特記事項 : このクラスは同じ引数を持ち同様の振る舞いをします。",  # noqa
    ##################################################
    "## ForDictKeys API": "## ForDictKeys API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "The loop implementation class for the `ap.Dictionary` keys.<hr>": "`ap.Dictionary`の各キーのためのループ用のクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `dict_`: Dictionary[_DictKey, Any]": "- `dict_`: Dictionary[_DictKey, Any]",
    ##################################################
    "  - A dictionary to iterate.": "  - イテレーションで扱うための辞書。",
    ##################################################
    "- `dict_key_type`: Type[_DictKey]": "- `dict_key_type`: Type[_DictKey]",
    ##################################################
    "  - A dictionary key type. This interface accepts hashable types, such as the `String`, `Int`, `Number`, or `Boolean`.": "  - 辞書のキーの型。このインターフェイスは`String`、`Int`、`Number`、`Boolean`といったハッシュ化可能な型を受け付けます。",  # noqa
    ##################################################
    "- `locals_`: Optional[Dict[str, Any]], optional": "- `locals_`: Optional[Dict[str, Any]], optional",  # noqa
    ##################################################
    "  - Current scope's local variables. Set locals() value to this argument. If specified, this interface reverts all local scope VariableNameMixIn variables (like Int, Sprite) at the end of a with-statement scope. This setting is useful when you don't want to update each variable.": "  - 現在のスコープの各ローカル変数。この引数にはlocals()関数の返却値を設定してください。もしこの引数が指定された場合、このインターフェイスはローカルスコープのVariableNameMixInの各変数（例 : IntやSpriteなど）の値をwithステートメントの最後で復元します。この設定は各変数を更新したくない場合等に役立ちます。",  # noqa
    ##################################################
    "- `globals_`: Optional[Dict[str, Any]], optional": "- `globals_`: Optional[Dict[str, Any]], optional",  # noqa
    ##################################################
    "  - Current scope's global variables. Set globals() value to this argument. This setting works the same way as the locals_ argument.": "  - 現在のスコープの各グローバル変数。設定する場合にはglobal()関数の値をこの引数に指定してください。この設定はlocals_引数と同じように動作します。",  # noqa
    ##################################################
    "- `variable_name_suffix`: str, optional": "- `variable_name_suffix`: str, optional",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> dict_: ap.Dictionary[ap.String, ap.Boolean] = ap.Dictionary(\n...     {\n...         ap.String("apple"): ap.Boolean(True),\n...         ap.String("orange"): ap.Boolean(False),\n...     }\n... )\n>>> keys: ap.Array[ap.String] = ap.Array([])\n>>> with ap.ForDictKeys(dict_=dict_, dict_key_type=ap.String) as key:\n...     keys.append(key)\n...\n>>> ap.assert_arrays_equal(\n...     keys,\n...     ["apple", "orange"],\n... )\n```': '```py\n>>> import apysc as ap\n>>> dict_: ap.Dictionary[ap.String, ap.Boolean] = ap.Dictionary(\n...     {\n...         ap.String("apple"): ap.Boolean(True),\n...         ap.String("orange"): ap.Boolean(False),\n...     }\n... )\n>>> keys: ap.Array[ap.String] = ap.Array([])\n>>> with ap.ForDictKeys(dict_=dict_, dict_key_type=ap.String) as key:\n...     keys.append(key)\n...\n>>> ap.assert_arrays_equal(\n...     keys,\n...     ["apple", "orange"],\n... )\n```',  # noqa
}



"""The class implementation for the `thumb_down_off_alt` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialThumbDownOffAltIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `thumb_down_off_alt` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M24 24H0V0h24v24z" fill="none"/><path d="M10.89 18.28l.57-2.89c.12-.59-.04-1.2-.42-1.66-.38-.46-.94-.73-1.54-.73H4v-1.08L6.57 6h8.09c.18 0 .34.16.34.34v7.84l-4.11 4.1M10 22l6.41-6.41c.38-.38.59-.89.59-1.42V6.34C17 5.05 15.95 4 14.66 4h-8.1c-.71 0-1.36.37-1.72.97l-2.67 6.15c-.11.25-.17.52-.17.8V13c0 1.1.9 2 2 2h5.5l-.92 4.65c-.05.22-.02.46.08.66.23.45.52.86.88 1.22L10 22zm10-7h2V4h-2c-.55 0-1 .45-1 1v9c0 .55.45 1 1 1z"/></svg>'  # noqa



"""The class implementation for the `text_rotation_angledown` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialTextRotationAngledownIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `text_rotation_angledown` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M19.4 4.91l-1.06-1.06L7.2 8.27l1.48 1.48 2.19-.92 3.54 3.54-.92 2.19 1.48 1.48L19.4 4.91zm-6.81 3.1l4.87-2.23-2.23 4.87-2.64-2.64zM14.27 21v-4.24l-1.41 1.41-8.84-8.84-1.42 1.42 8.84 8.84L10.03 21h4.24z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: range.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# range function": "# range 関数",
    ##################################################
    "This page explains the `range` function.": "このページでは`range`関数について説明していきます。",
    ##################################################
    "## What function is this?": "## 関数概要",
    ##################################################
    "The `range` function creates a range array of `ap.Int` (e.g., `[0, 1, 2, 3, 4, 5]`).": "`range`関数は`ap.Int`型の指定の範囲の配列を作成します（例 : `[0, 1, 2, 3, 4, 5]`）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "If you specify only one argument, a range array becomes 0 to argument value -1.": "もしも引数を1つだけ指定した場合、範囲の配列は0から指定された引数の値を-1した範囲になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrange_arr: ap.Array[ap.Int] = ap.range(5)\nap.assert_equal(range_arr, [0, 1, 2, 3, 4])\nap.save_overall_html(dest_dir_path="range_basics_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrange_arr: ap.Array[ap.Int] = ap.range(5)\nap.assert_equal(range_arr, [0, 1, 2, 3, 4])\nap.save_overall_html(dest_dir_path="range_basics_usage_1/")\n```',  # noqa
    ##################################################
    "Also, if you specify two arguments, a range array becomes a first argument value to a second argument value -1.": "また、もし2つの引数を指定した場合には範囲の配列は最初の引数の値～2つ目の引数の値を-1した範囲になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrange_arr: ap.Array[ap.Int] = ap.range(2, 4)\nap.assert_equal(range_arr, [2, 3])\nap.save_overall_html(dest_dir_path="range_basics_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrange_arr: ap.Array[ap.Int] = ap.range(2, 4)\nap.assert_equal(range_arr, [2, 3])\nap.save_overall_html(dest_dir_path="range_basics_usage_2/")\n```',  # noqa
    ##################################################
    "If three arguments, a range array becomes a first argument value to a second argument value, and a step between each value becomes a third argument value.": "もし3つ引数を指定した場合には範囲の配列は最初の引数の値～2つ目の引数の値を-1した範囲となり、各値は3つ目に指定された引数の値のステップで設定されます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrange_arr: ap.Array[ap.Int] = ap.range(2, 10, 2)\nap.assert_equal(range_arr, [2, 4, 6, 8])\nap.save_overall_html(dest_dir_path="range_basics_usage_3/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=0,\n    stage_height=0,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrange_arr: ap.Array[ap.Int] = ap.range(2, 10, 2)\nap.assert_equal(range_arr, [2, 4, 6, 8])\nap.save_overall_html(dest_dir_path="range_basics_usage_3/")\n```',  # noqa
    ##################################################
    "## range function API": "## range 関数のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Create a range array of integers.<hr>": "整数の指定範囲の配列を生成します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `arr`: Array[Int]": "- `arr`: Array[Int]",
    ##################################################
    "  - A created array.": "  - 生成された配列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> range_arr: ap.Array[ap.Int] = ap.range(5)\n>>> ap.assert_equal(range_arr, [0, 1, 2, 3, 4])\n>>> range_arr = ap.range(2, 4)\n>>> ap.assert_equal(range_arr, [2, 3])\n>>> range_arr = ap.range(2, 10, 2)\n>>> ap.assert_equal(range_arr, [2, 4, 6, 8])\n```": "```py\n>>> import apysc as ap\n>>> range_arr: ap.Array[ap.Int] = ap.range(5)\n>>> ap.assert_equal(range_arr, [0, 1, 2, 3, 4])\n>>> range_arr = ap.range(2, 4)\n>>> ap.assert_equal(range_arr, [2, 3])\n>>> range_arr = ap.range(2, 10, 2)\n>>> ap.assert_equal(range_arr, [2, 4, 6, 8])\n```",  # noqa
}



"""The class implementation for the outlined `qr_code_scanner` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialQrCodeScannerOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `qr_code_scanner` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><path d="M9.5,6.5v3h-3v-3H9.5 M11,5H5v6h6V5L11,5z M9.5,14.5v3h-3v-3H9.5 M11,13H5v6h6V13L11,13z M17.5,6.5v3h-3v-3H17.5 M19,5h-6v6 h6V5L19,5z M13,13h1.5v1.5H13V13z M14.5,14.5H16V16h-1.5V14.5z M16,13h1.5v1.5H16V13z M13,16h1.5v1.5H13V16z M14.5,17.5H16V19h-1.5 V17.5z M16,16h1.5v1.5H16V16z M17.5,14.5H19V16h-1.5V14.5z M17.5,17.5H19V19h-1.5V17.5z M22,7h-2V4h-3V2h5V7z M22,22v-5h-2v3h-3v2 H22z M2,22h5v-2H4v-3H2V22z M2,2v5h2V4h3V2H2z"/></svg>'  # noqa



"""The class implementation for the outlined `fiber_new` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFiberNewOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `fiber_new` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M7.25 12.5L4.75 9H3.5v6h1.25v-3.5L7.3 15h1.2V9H7.25zM9.5 15h4v-1.25H11v-1.11h2.5v-1.26H11v-1.12h2.5V9h-4zm9.75-6v4.5h-1.12V9.99h-1.25v3.52h-1.13V9H14.5v5c0 .55.45 1 1 1h4c.55 0 1-.45 1-1V9h-1.25z"/></svg>'  # noqa



"""The class implementation for the `volume_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVolumeOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `volume_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M16.5 12c0-1.77-1.02-3.29-2.5-4.03v2.21l2.45 2.45c.03-.2.05-.41.05-.63zm2.5 0c0 .94-.2 1.82-.54 2.64l1.51 1.51C20.63 14.91 21 13.5 21 12c0-4.28-2.99-7.86-7-8.77v2.06c2.89.86 5 3.54 5 6.71zM4.27 3L3 4.27 7.73 9H3v6h4l5 5v-6.73l4.25 4.25c-.67.52-1.42.93-2.25 1.18v2.06c1.38-.31 2.63-.95 3.69-1.81L19.73 21 21 19.73l-9-9L4.27 3zM12 4L9.91 6.09 12 8.18V4z"/></svg>'  # noqa



"""Class implementation for the scale-x from the given point
animation value.
"""

from typing import Generic
from typing import TypeVar
from typing import Union

from typing_extensions import final

from apysc._animation.animation_base import AnimationBase
from apysc._animation.easing import Easing
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.variable_name_mixin import VariableNameMixIn

_Target = TypeVar("_Target", bound=VariableNameMixIn)


class AnimationScaleXFromPoint(AnimationBase[_Target], Generic[_Target]):
    """
    The animation class for a scale-x from the given point.

    References
    ----------
    - animation_scale_x_from_point interface
        - https://simon-ritchie.github.io/apysc/en/animation_scale_x_and_y_from_point.html  # noqa
    - Animation interfaces duration setting
        - https://simon-ritchie.github.io/apysc/en/animation_duration.html
    - Animation interfaces delay setting
        - https://simon-ritchie.github.io/apysc/en/animation_delay.html
    - Each animation interface return value
        - https://simon-ritchie.github.io/apysc/en/animation_return_value.html  # noqa
    - Sequential animation setting
        - https://simon-ritchie.github.io/apysc/en/sequential_animation.html
    - animation_parallel interface
        - https://simon-ritchie.github.io/apysc/en/animation_parallel.html
    - Easing enum
        - https://simon-ritchie.github.io/apysc/en/easing_enum.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50
    ... )
    >>> animation: ap.AnimationScaleXFromPoint
    >>> animation = rectangle.animation_scale_x_from_point(
    ...     scale_x_from_point=0.5,
    ...     x=ap.Number(100),
    ...     duration=1500,
    ...     easing=ap.Easing.EASE_OUT_QUINT,
    ... )
    >>> _ = animation.start()
    """

    _scale_x_from_point: Number
    _x: Number
    _before_scale_x_from_point: Number
    _scale_x_from_point_diff_ratio: Number

    @final
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        target: _Target,
        scale_x_from_point: Union[float, Number],
        x: Union[float, Number],
        duration: Union[int, Int] = 3000,
        delay: Union[int, Int] = 0,
        easing: Easing = Easing.LINEAR,
    ) -> None:
        """
        The animation class for a scale-x from the given point.

        Parameters
        ----------
        target : ScaleXFromPointMixIn
            A target instance of the animation target
            (e.g., `Rectangle` instance).
        scale_x_from_point : float or Number
            The final scale-x from the given point of the animation.
        x : float or Number
            X-coordinate.
        duration : int or Int, default 3000
            Milliseconds before an animation ends.
        delay : int or Int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Raises
        ------
        TypeError
            If a specified target is not a ScaleXFromPointMixIn
            instance.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.ANIMATION_SCALE_X_FROM_POINT
        )
        self._x = to_apysc_val_from_builtin.get_copied_number_from_builtin_val(
            float_or_num=x
        )
        target_: VariableNameMixIn = target
        if isinstance(target_, ScaleXFromPointMixIn):
            target_._initialize_scale_x_from_point_if_not_initialized()
            self._before_scale_x_from_point = target_.get_scale_x_from_point(x=self._x)
        else:
            raise TypeError(
                "Specified `target` argument is not a "
                f"ScaleXFromPointMixIn: {type(target_)}"
            )
        self._scale_x_from_point = (
            to_apysc_val_from_builtin.get_copied_number_from_builtin_val(
                float_or_num=scale_x_from_point
            )
        )
        one: Number = Number(1.0)
        self._scale_x_from_point_diff_ratio = (
            one / self._before_scale_x_from_point * self._scale_x_from_point
        )
        self._set_basic_animation_settings(
            target=target, duration=duration, delay=delay, easing=easing
        )
        super(AnimationScaleXFromPoint, self).__init__(variable_name=variable_name)

    @final
    def _get_animation_func_expression(self) -> str:
        """
        Get a animation function expression.

        Returns
        -------
        expression : str
            Animation function expression.
        """
        from apysc._type import value_util

        diff_ratio_str: str = value_util.get_value_str_for_expression(
            value=self._scale_x_from_point_diff_ratio
        )
        x_str: str = value_util.get_value_str_for_expression(value=self._x)
        return f"\n  .scale({diff_ratio_str}, 1, {x_str}, 0);"

    @final
    def _get_complete_event_in_handler_head_expression(self) -> str:
        """
        Get an expression to be inserted into the complete event
        handler's head.

        Returns
        -------
        expression : str
            An expression to insert into the complete event
            handler's head.
        """
        from apysc._display import scale_interface_helper
        from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
        from apysc._type import value_util

        expression: str = ""
        if isinstance(self._target, ScaleXFromPointMixIn):
            self._target._initialize_scale_x_from_point_if_not_initialized()
            key_exp_str: str = scale_interface_helper.get_coordinate_key_for_expression(
                coordinate=self._x
            ).value
            target_scale_x_str: str = value_util.get_value_str_for_expression(
                value=self._target._scale_x_from_point
            )
            scale_x_str: str = value_util.get_value_str_for_expression(
                value=self._scale_x_from_point
            )
            expression = f"{target_scale_x_str}[{key_exp_str}] = " f"{scale_x_str};"
        return expression



# mypy: disable-error-code=type-var

"""The decorator implementation for the auto-reloading.
"""

import functools
import os
import time
from datetime import datetime
from datetime import timedelta
from typing import Any
from typing import Callable
from typing import List
from typing import Optional
from typing import TypeVar

# pyright: reportInvalidTypeVarUse=false
_Callable = TypeVar("_Callable", bound=Callable)


def set_auto_reloading(
    *,
    checking_dir_paths: List[str],
    max_checking_num: Optional[int] = None,
) -> _Callable:
    """
    Set an auto-reloading setting as a decorator.

    Notes
    -----
    Currently, this setting checks only `.py` files.

    Parameters
    ----------
    checking_dir_paths : List[str]
        Directory paths to check for file changes.
    max_checking_num : Optional[int], optional
        Maximum checking attempts number.
        Mainly this interface uses this setting for the testing.

    Returns
    -------
    wrapped : Callable
        Wrapped callable object.
    """

    def wrapped(callable_: _Callable) -> _Callable:
        @functools.wraps(callable_)
        def inner_wrapped(*args: Any, **kwargs: Any) -> Any:
            result: Any = callable_(*args, **kwargs)
            last_executed_time: datetime = datetime.now()
            time.sleep(1.0)

            checking_count: int = 0
            while True:
                if max_checking_num is not None and checking_count >= max_checking_num:
                    break
                if not _are_files_updated(
                    last_executed_time=last_executed_time,
                    checking_dir_paths=checking_dir_paths,
                ):
                    time.sleep(1.0)
                    checking_count += 1
                    continue
                try:
                    result = callable_(*args, **kwargs)
                    last_executed_time = datetime.now()
                    time.sleep(1.0)
                except Exception:
                    pass
                finally:
                    checking_count += 1
            return result

        return inner_wrapped  # type: ignore

    return wrapped  # type: ignore


def _are_files_updated(
    *,
    last_executed_time: datetime,
    checking_dir_paths: List[str],
) -> bool:
    """
    Get a boolean, whether files are updated or not.

    Parameters
    ----------
    last_executed_time : datetime
        A last execution time.
    checking_dir_paths : List[str]
        Directory paths to check for file changes.

    Returns
    -------
    result : bool
        If any updated files exist, this interface returns True.
    """
    timedelta_: timedelta = timedelta(seconds=1)
    for dire_path in checking_dir_paths:
        file_or_dir_names: List[str] = os.listdir(dire_path)
        for file_or_dir_name in file_or_dir_names:
            file_or_dir_path: str = os.path.join(dire_path, file_or_dir_name)
            if os.path.isdir(file_or_dir_path):
                result: bool = _are_files_updated(
                    last_executed_time=last_executed_time,
                    checking_dir_paths=[file_or_dir_path],
                )
                if result:
                    return True
                continue

            if not file_or_dir_path.endswith(".py"):
                continue

            last_modified: datetime = datetime.fromtimestamp(
                os.stat(file_or_dir_path).st_mtime,
            )
            if last_modified >= last_executed_time - timedelta_:
                return True
    return False



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_line_thickness.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase line_thickness interface": "# GraphicsBase クラスの line_thickness インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class `line_thickness` property interface.": "このページでは`GraphicsBase`クラスの`line_thickness`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `line_thickness` property interface updates or get the instance's line thickness (line width).": "`line_thickness`属性はインスタンスの線幅の更新もしくは取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) the `Int` value.": "getterもしくはsetterの各インターフェイスの値は`Int`型の値になります。",  # noqa
    ##################################################
    "The following example sets the 5-pixel line thickness to the first rectangle and the 10-pixel line thickness to the second one:": "以下のコード例では1つ目の四角に5pxの線幅を設定しており、2つ目の四角には10pxの線幅を設定しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=1)\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_1.line_thickness = ap.Int(5)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_2.line_thickness = ap.Int(10)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_thickness_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=1)\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle_1.line_thickness = ap.Int(5)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_2.line_thickness = ap.Int(10)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_thickness_basic_usage/")\n```',  # noqa
    ##################################################
    "## line_thickness property API": "## line_thickness 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get this instance's line thickness.<hr>": "このインスタンスの線幅を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line_thickness`: Int": "- `line_thickness`: Int",
    ##################################################
    "  - Current line thickness.": "  - 現在の線幅。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_thickness\nInt(5)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_thickness\nInt(5)\n```',  # noqa
}



"""The mix-in class implementation for the `slice` method.
"""

from typing import TYPE_CHECKING
from typing import Optional
from typing import Union
from typing import cast

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._validation import arg_validation_decos

if TYPE_CHECKING:
    from apysc._type.int import Int
    from apysc._type.string import String


class StringSliceMixIn:
    @final
    @arg_validation_decos.is_apysc_string(arg_position_index=0)
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_integer(arg_position_index=2, optional=True)
    @arg_validation_decos.is_builtin_string(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def slice(
        self,
        *,
        start: Union[int, "Int"],
        end: Optional[Union[int, "Int"]] = None,
        variable_name_suffix: str = "",
    ) -> "String":
        """
        Get a sliced string based on the specified arguments range.

        Parameters
        ----------
        start : Union[int, "Int"]
            A start index of the slice range.
        end : Optional[Union[int, "Int"]], optional
            An end index of the slice range. If this argument is
            not specified, this method skips the end position's slicing.
        variable_name_suffix : str, default ""
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        Returns
        -------
        result : String
            A sliced result string.

        References
        ----------
        - String class slice method
            - https://simon-ritchie.github.io/apysc/en/string_slice.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     stage_width=0,
        ...     stage_height=0,
        ...     background_color=ap.Color("#333"),
        ...     stage_elem_id="stage",
        ... )

        >>> string: ap.String = ap.String("012345")
        >>> result_string: ap.String = string.slice(start=0)
        >>> result_string
        String("012345")

        >>> result_string = string.slice(start=1)
        >>> result_string
        String("12345")

        >>> result_string = string.slice(start=0, end=2)
        >>> result_string
        String("01")

        >>> result_string = string.slice(start=2, end=4)
        >>> result_string
        String("23")

        >>> result_string = string.slice(start=-2)
        >>> result_string
        String("45")

        >>> result_string = string.slice(start=-3, end=-1)
        >>> result_string
        String("34")
        """
        from apysc._converter.to_apysc_val_from_builtin import (
            get_copied_int_from_builtin_val,
        )
        from apysc._expression import expression_data_util
        from apysc._type.int import Int
        from apysc._type.string import String
        from apysc._validation.string_validation import validate_apysc_string_type

        result: String = String("", variable_name_suffix=variable_name_suffix)
        self_string: String = validate_apysc_string_type(string=self)
        start_ap_int: Int = get_copied_int_from_builtin_val(integer=start)
        end_ap_int: Optional[Int] = None
        if end is not None:
            end_ap_int = get_copied_int_from_builtin_val(integer=end)
            result._value = self_string._value[
                start_ap_int._value : cast(Int, end_ap_int)._value
            ]
        else:
            result._value = self_string._value[start_ap_int._value :]

        expression: str = f"{result.variable_name} = {self_string.variable_name}.slice("
        if end_ap_int is not None:
            expression += f"{start_ap_int.variable_name}, {end_ap_int.variable_name}"
        else:
            expression += f"{start_ap_int.variable_name}"
        expression += ");"
        expression_data_util.append_js_expression(expression=expression)

        return result



"""The class implementation for the outlined `mark_chat_read` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMarkChatReadOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `mark_chat_read` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/><path d="M12,18l-6,0l-4,4V4c0-1.1,0.9-2,2-2h16c1.1,0,2,0.9,2,2v7l-2,0V4H4v12l8,0V18z M23,14.34l-1.41-1.41l-4.24,4.24l-2.12-2.12 l-1.41,1.41L17.34,20L23,14.34z"/></g></svg>'  # noqa



"""The class implementation for the `loyalty` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLoyaltyIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `loyalty` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M21.41 11.58l-9-9C12.05 2.22 11.55 2 11 2H4c-1.1 0-2 .9-2 2v7c0 .55.22 1.05.59 1.42l9 9c.36.36.86.58 1.41.58.55 0 1.05-.22 1.41-.59l7-7c.37-.36.59-.86.59-1.41 0-.55-.23-1.06-.59-1.42zM5.5 7C4.67 7 4 6.33 4 5.5S4.67 4 5.5 4 7 4.67 7 5.5 6.33 7 5.5 7zm11.77 8.27L13 19.54l-4.27-4.27C8.28 14.81 8 14.19 8 13.5c0-1.38 1.12-2.5 2.5-2.5.69 0 1.32.28 1.77.74l.73.72.73-.73c.45-.45 1.08-.73 1.77-.73 1.38 0 2.5 1.12 2.5 2.5 0 .69-.28 1.32-.73 1.77z"/></svg>'  # noqa



"""The class implementation for the outlined `library_add_check` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialLibraryAddCheckOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `library_add_check` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><path d="M20,4v12H8V4H20 M20,2H8C6.9,2,6,2.9,6,4v12c0,1.1,0.9,2,2,2h12c1.1,0,2-0.9,2-2V4C22,2.9,21.1,2,20,2L20,2z M12.47,14 L9,10.5l1.4-1.41l2.07,2.08L17.6,6L19,7.41L12.47,14z M4,6H2v14c0,1.1,0.9,2,2,2h14v-2H4V6z"/><path d="M0,0h24v24H0V0z" fill="none"/></g></svg>'  # noqa



"""The class implementation for the `mediation` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialMediationIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `mediation` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M22 12l-4 4-1.41-1.41L18.17 13h-5.23c-.34 3.1-2.26 5.72-4.94 7.05C7.96 21.69 6.64 23 5 23c-1.66 0-3-1.34-3-3s1.34-3 3-3c.95 0 1.78.45 2.33 1.14 1.9-1.03 3.26-2.91 3.58-5.14h-3.1C7.4 14.16 6.3 15 5 15c-1.66 0-3-1.34-3-3s1.34-3 3-3c1.3 0 2.4.84 2.82 2h3.1c-.32-2.23-1.69-4.1-3.59-5.14C6.78 6.55 5.95 7 5 7 3.34 7 2 5.66 2 4s1.34-3 3-3c1.64 0 2.96 1.31 2.99 2.95 2.68 1.33 4.6 3.95 4.94 7.05h5.23l-1.58-1.59L18 8l4 4z"/></svg>'  # noqa



"""The class implementation for the `block` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBlockIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `block` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zM4 12c0-4.42 3.58-8 8-8 1.85 0 3.55.63 4.9 1.69L5.69 16.9C4.63 15.55 4 13.85 4 12zm8 8c-1.85 0-3.55-.63-4.9-1.69L18.31 7.1C19.37 8.45 20 10.15 20 12c0 4.42-3.58 8-8 8z"/></svg>'  # noqa
